#include <stdio.h>
#include <windows.h>   
#include "vsm.hpp"   

/********************************************************************/

class km41c64 : public IACTIVEMODEL,public IDSIMMODEL 
{ 
public: 
  int isdigital (CHAR *pinname); 
  void setup (IINSTANCE *inst, IDSIMCKT *dsim); 
  void runctrl (RUNMODES mode); 
  void actuate (REALTIME time, ACTIVESTATE newstate); 
  BOOL indicate (REALTIME time, ACTIVEDATA *data); 
  void simulate (ABSTIME time, DSIMMODES mode); 
  void callback (ABSTIME time, EVENTID eventid); 
  void initialize (ICOMPONENT *cpt); 
  ISPICEMODEL *getspicemodel (CHAR *device); 
  IDSIMMODEL *getdsimmodel (CHAR *device); 
  void plot (ACTIVESTATE state); 
  void animate (INT element, ACTIVEDATA *newstate); 
  BOOL actuate (WORD key, INT x, INT y, DWORD flags); 
  unsigned short Addr_latch_ras;
  unsigned short Addr_latch_cas;
  unsigned char data[8196];
  unsigned char bit_pos[8];

private: 
 unsigned char get_addr();
 unsigned char get_data();
 void set_data(ABSTIME time,unsigned char tmp);
 void set_data_Z(ABSTIME time);

  IINSTANCE *instance;  
  IDSIMCKT *ckt;      
  ICOMPONENT *component;   
  IDSIMPIN *pin_we; 
  IDSIMPIN *pin_cas; 
  IDSIMPIN *pin_ras; 
  IDSIMPIN *pin_A0; 
  IDSIMPIN *pin_A1; 
  IDSIMPIN *pin_A2; 
  IDSIMPIN *pin_A3; 
  IDSIMPIN *pin_A4; 
  IDSIMPIN *pin_A5; 
  IDSIMPIN *pin_A6; 
  IDSIMPIN *pin_A7; 
  IDSIMPIN *pin_DI; 
  IDSIMPIN *pin_DO; 
unsigned char tmpA;
}; 

HANDLE hProcessHeap;

extern "C" IDSIMMODEL __declspec(dllexport)*createdsimmodel(CHAR *device,ILICENCESERVER *ils)   {     
        ils->authorize(0x88888888, 0x69);  
        return new km41c64;   
}   

extern "C" VOID __declspec(dllexport)deletedsimmodel(IDSIMMODEL*model)   {   
    delete (km41c64*)model; 
}   
       
int km41c64::isdigital(CHAR *pinname)   {   
        return 1;   
}   

/********************************************************************/

void km41c64::setup(IINSTANCE *inst,IDSIMCKT *dsim)   {   
    instance = inst;    
          ckt = dsim;     
        
    instance->log("km41c64.dll - Version 1");
//    instance->log("START");
    hProcessHeap=GetProcessHeap();
  
    pin_we = instance->getdsimpin("$WE$", true); pin_we->setstate(0,1,FLT);   
    pin_cas = instance->getdsimpin("$CAS$", true);pin_cas->setstate(0,1,FLT);   
    pin_ras = instance->getdsimpin("$RAS$", true);pin_ras->setstate(0,1,FLT);   
    pin_A0 = instance->getdsimpin("A0", true);   pin_A0->setstate(0,1,FLT);   
    pin_A1 = instance->getdsimpin("A1", true);   pin_A1->setstate(0,1,FLT);   
    pin_A2 = instance->getdsimpin("A2", true);   pin_A2->setstate(0,1,FLT);   
    pin_A3 = instance->getdsimpin("A3", true);   pin_A3->setstate(0,1,FLT);   
    pin_A4 = instance->getdsimpin("A4", true);   pin_A4->setstate(0,1,FLT);   
    pin_A5 = instance->getdsimpin("A5", true);   pin_A5->setstate(0,1,FLT);   
    pin_A6 = instance->getdsimpin("A6", true);   pin_A6->setstate(0,1,FLT);   
    pin_A7 = instance->getdsimpin("A7", true);   pin_A7->setstate(0,1,FLT);
    pin_DI = instance->getdsimpin("DI", true);   pin_DI->setstate(0,1,FLT);
    pin_DO = instance->getdsimpin("DO", true);   pin_DO->setstate(0,1,FLT);

    bit_pos[0]=0x01; 
    bit_pos[1]=0x02; 
    bit_pos[2]=0x04; 
    bit_pos[3]=0x08; 
    bit_pos[4]=0x10; 
    bit_pos[5]=0x20; 
    bit_pos[6]=0x40; 
    bit_pos[7]=0x80; 
}   
       
void km41c64::runctrl(RUNMODES mode)  {   

}   
       
       
void km41c64::actuate(REALTIME time,ACTIVESTATE newstate) {   

}   
       
BOOL km41c64::indicate(REALTIME time,ACTIVEDATA *data)   {      
  return TRUE;
}   
       
void km41c64::set_data_Z(ABSTIME time)
{
 pin_DO->setstate(time,1000,FLT);   
}


unsigned char km41c64::get_addr()
{
unsigned char tmp=0;
 if (ishigh(pin_A0->istate())) tmp|=0x01;
 if (ishigh(pin_A1->istate())) tmp|=0x02;
 if (ishigh(pin_A2->istate())) tmp|=0x04;
 if (ishigh(pin_A3->istate())) tmp|=0x08;
 if (ishigh(pin_A4->istate())) tmp|=0x10;
 if (ishigh(pin_A5->istate())) tmp|=0x20;
 if (ishigh(pin_A6->istate())) tmp|=0x40;
 if (ishigh(pin_A7->istate())) tmp|=0x80;
 return tmp;
}

unsigned char km41c64::get_data()
{
unsigned char tmp=0;
 if (ishigh(pin_DI->istate())) tmp=0xFF;
 return tmp;
}

void km41c64::set_data(ABSTIME time,unsigned char tmp)
{
 if (tmp==0) pin_DO->setstate(time,1,SLO); else pin_DO->setstate(time,1,SHI);
}

void km41c64::simulate (ABSTIME time,DSIMMODES mode)
{
unsigned short tmp_addr;
 if (pin_we->isnegedge()&pin_cas->isinactive()&pin_ras->isinactive()) set_data_Z(time); // WE-\_ & CAS=0 & RAS=0
 if (pin_ras->isnegedge()) 
  {
   tmpA = Addr_latch_ras;
   Addr_latch_ras=get_addr();
  }
 if (pin_cas->isnegedge()&pin_ras->isinactive()) // CAS-\_ & RAS=0
  {
   tmpA = Addr_latch_cas;
   Addr_latch_cas=get_addr();
   if (pin_we->isinactive()) set_data_Z(time); //(CAS-\_ & RAS=0) & WE=0
     else 
     { //(CAS-\_ & RAS=0) & WE=1  out data from ram
      tmp_addr=Addr_latch_cas*256+Addr_latch_ras;
      set_data(time,data[tmp_addr/8]&bit_pos[tmp_addr&0x07]);
     }
  }
 if (pin_cas->isactive()&pin_ras->isinactive()) set_data_Z(time); //CAS=1 & RAS=0
 if (pin_ras->isinactive()) //RAS=0 & ((CAS_/- & WE=0) | (CAS=0 & WE_/-))
  if ((pin_cas->isposedge()&pin_we->isinactive())|(pin_cas->isinactive()&pin_we->isposedge()))
  { // in data to ram
   tmp_addr=Addr_latch_cas*256+Addr_latch_ras;
   data[tmp_addr/8]&=~bit_pos[tmp_addr&0x07];
   data[tmp_addr/8]|=get_data()&bit_pos[tmp_addr&0x07];
 //  instance->log("km41c64 WR Addr %x  data()=%x",Addr_latch_cas*256+Addr_latch_ras ,data[Addr_latch_cas*256+Addr_latch_ras]);
  }
}
      
void km41c64::callback(ABSTIME time,EVENTID eventid)   {   

}   
              
extern "C" IACTIVEMODEL __declspec(dllexport)*createactivemodel(CHAR *device,ILICENCESERVER *ils)  {    
        ils->authorize (0);     
        return new km41c64;   
}   
              
extern "C" VOID  __declspec(dllexport)deleteactivemodel (IACTIVEMODEL *model)  {    
    
    
  delete (km41c64*)model;   
}   
               
void km41c64::initialize(ICOMPONENT *cpt)  {    

}   
               
ISPICEMODEL *km41c64::getspicemodel(CHAR *)  {    
        return NULL;    
}   
       
IDSIMMODEL  *km41c64::getdsimmodel(CHAR *)  {      
        return this;    
}   

void km41c64::plot(ACTIVESTATE state)  {   

//  MessageBox(0, "plot", "Break", MB_OK);
}
        
void km41c64::animate(INT element, ACTIVEDATA *data)  {    
//  MessageBox(0, "animate", "Break", MB_OK);

}   

BOOL km41c64::actuate(WORD key, INT x, INT y, DWORD flags)  {
//  MessageBox(0, "ACTUATE2", "Break", MB_OK);
        return FALSE;    
}  
